@@ -36,14 +36,25 @@ from uuid import uuid4
 from bokeh.models import Button, TextInput, Div, Range1d, LinearAxis, CustomJS, Spacer
 from bokeh.plotting import ColumnDataSource, figure, show
 from bokeh.layouts import column, row, Spacer
-from ..utils import resource_manager
+from bokeh.io import reset_output as reset_bokeh_output
+from ..utils import resource_manager, reset_resource_manager
 
 try:
+    ## gclean version number needed for proper interactive clean behavior
+    from casatasks.private.imagerhelpers._gclean import _GCV001
     from casatasks.private.imagerhelpers._gclean import gclean as _gclean
 except:
-    _gclean = None
-    from casagui.utils import warn_import
-    warn_import('casatasks')
+    try:
+        ###
+        ### enable this warning when casa6 a usable _gclean.py (i.e. compatibility is not the default)
+        ###
+        #print('warning: using tclean compatibility layer...')
+        from ..private.compatibility.casatasks.private.imagerhelpers._gclean import _GCV001
+        from ..private.compatibility.casatasks.private.imagerhelpers._gclean import gclean as _gclean
+    except:
+        _gclean = None
+        from casagui.utils import warn_import
+        warn_import('casatasks')
 
 from casagui.utils import find_ws_address, convert_masks
 from casagui.toolbox import CubeMask
@@ -141,22 +152,48 @@ class InteractiveClean:
 
     '''
     def __stop( self ):
-        resource_manager.stop_asyncio_loop()
+        resource_manager( ).stop_asyncio_loop()
         if self._control_server is not None and self._control_server.ws_server.is_serving( ):
-            resource_manager.stop_asyncio_loop()
+            resource_manager( ).stop_asyncio_loop()
         if self._converge_server is not None and self._converge_server.ws_server.is_serving( ):
-            resource_manager.stop_asyncio_loop()
+            resource_manager( ).stop_asyncio_loop()
 
     def _abort_handler( self, loop, err ):
         self._error_result = err
         self.__stop( )
 
-    def __init__( self, vis, imagename, mask=None, field='', spw='', timerange='', uvrange='', antenna='', scan='', intent='', datacolumn='corrected',
-                  nterms=int(2), imsize=[100], cell=[ ], phasecenter='', stokes='I', specmode='cube', reffreq='', nchan=-1, start='', width='',
-                  outframe='LSRK', interpolation='linear', perchanweightdensity=True, gridder='standard', wprojplanes=int(1), mosweight=True,
-                  psterm=False, wbawp=True, usepointing=False, conjbeams=False, pointingoffsetsigdev=[  ], pblimit=0.2, deconvolver='hogbom',
-                  niter=0, threshold='0.1Jy', nsigma=0.0, cycleniter=-1, cyclefactor=1.0, scales=[], restoringbeam='', pbcor=False,
-                  weighting='natural', robust=float(0.5), npixels=0, gain=float(0.1), sidelobethreshold=3.0, noisethreshold=5.0,
+    def __reset( self ):
+        if self.__pipes_initialized:
+            self._pipe = { 'control': None, 'converge': None }
+            reset_bokeh_output( )
+            reset_resource_manager( )
+            self._clean.reset( )
+
+        ###
+        ### used by data pipe (websocket) initialization function
+        ###
+        self.__pipes_initialized = False
+        self._mask_history = [ ]
+
+        self._cube = CubeMask( self._residual_path, mask=self._mask_path, abort=self._abort_handler )
+        ###
+        ### error or exception result
+        ###
+        self._error_result = None
+
+        ###
+        ### websocket servers
+        ###
+        self._control_server = None
+        self._converge_server = None
+
+
+    def __init__( self, vis, imagename, mask=None, field='', spw='', timerange='', uvrange='', antenna='', scan='', observation='', intent='',
+                  datacolumn='corrected', nterms=int(2), imsize=[100], cell=[ ], phasecenter='', stokes='I', startmodel='', specmode='cube', reffreq='',
+                  nchan=-1, start='', width='', outframe='LSRK', interpolation='linear', perchanweightdensity=True, gridder='standard', wprojplanes=int(1),
+                  mosweight=True, psterm=False, wbawp=True, usepointing=False, conjbeams=False, pointingoffsetsigdev=[  ], pblimit=0.2,
+                  deconvolver='hogbom', niter=0, threshold='0.1Jy', nsigma=0.0, cycleniter=-1, cyclefactor=1.0, scales=[], restoringbeam='',
+                  pbcor=False, weighting='natural', robust=float(0.5), npixels=0, gain=float(0.1), sidelobethreshold=3.0, noisethreshold=5.0,
                   lownoisethreshold=1.5, negativethreshold=0.0, minbeamfrac=0.3, growiterations=75, dogrowprune=True, minpercentchange=-1.0,
                   fastnoise=True, savemodel='none', parallel=False, nmajor=1 ):
 
@@ -164,10 +201,10 @@ class InteractiveClean:
             raise RuntimeError("deconvolver task does not support 'mtmf' deconvolver")
 
         ###
-        ### used by data pipe (websocket) initialization function
+        ### This is used to tell whether the websockets have been initialized, but also to
+        ### indicate if __call__ is being called multiple times to allow for resetting Bokeh
         ###
         self.__pipes_initialized = False
-        self._mask_history = [ ]
 
         ###
         ### color specs
@@ -198,12 +235,13 @@ class InteractiveClean:
             raise RuntimeError('casatasks gclean interface is not available')
 
         self._clean = _gclean( vis=vis, imagename=imagename, field=field, spw=spw, timerange=timerange,  uvrange=uvrange, antenna=antenna, scan=scan,
-                               intent=intent, datacolumn=datacolumn, nterms=nterms, imsize=imsize, cell=cell, phasecenter=phasecenter, stokes=stokes,
-                               specmode=specmode, reffreq=reffreq, nchan=nchan, start=start, width=width, outframe=outframe, interpolation=interpolation,
-                               perchanweightdensity=perchanweightdensity, gridder=gridder, wprojplanes=wprojplanes, mosweight=mosweight, psterm=psterm,
-                               wbawp=wbawp, usepointing=usepointing, conjbeams=conjbeams, pointingoffsetsigdev=pointingoffsetsigdev, pblimit=pblimit,
-                               deconvolver=deconvolver, niter=niter, threshold=threshold, nsigma=nsigma, cycleniter=cycleniter, cyclefactor=cyclefactor,
-                               scales=scales, restoringbeam=restoringbeam, pbcor=pbcor, weighting=weighting, robust=robust, npixels=npixels, gain=gain,
+                               observation=observation, intent=intent, datacolumn=datacolumn, nterms=nterms, imsize=imsize, cell=cell,
+                               phasecenter=phasecenter, stokes=stokes, startmodel=startmodel, specmode=specmode, reffreq=reffreq, nchan=nchan,
+                               start=start, width=width, outframe=outframe, interpolation=interpolation, perchanweightdensity=perchanweightdensity,
+                               gridder=gridder, wprojplanes=wprojplanes, mosweight=mosweight, psterm=psterm, wbawp=wbawp, usepointing=usepointing,
+                               conjbeams=conjbeams, pointingoffsetsigdev=pointingoffsetsigdev, pblimit=pblimit, deconvolver=deconvolver, niter=niter,
+                               threshold=threshold, nsigma=nsigma, cycleniter=cycleniter, cyclefactor=cyclefactor, scales=scales,
+                               restoringbeam=restoringbeam, pbcor=pbcor, weighting=weighting, robust=robust, npixels=npixels, gain=gain,
                                sidelobethreshold=sidelobethreshold, noisethreshold=noisethreshold, lownoisethreshold=lownoisethreshold,
                                negativethreshold=negativethreshold, minbeamfrac=minbeamfrac, growiterations=growiterations, dogrowprune=dogrowprune,
                                minpercentchange=minpercentchange, fastnoise=fastnoise, savemodel=savemodel, parallel=parallel, nmajor=nmajor,
@@ -222,10 +260,6 @@ class InteractiveClean:
         self._convergence_id = str(uuid4( ))
         #print(f'convergence:',self._convergence_id)
 
-        self._status['log'] = Div( text='''<hr style="width:790px">%s''' % ''.join([ f'<p style="width:790px">{cmd}</p>' for cmd in self._clean.cmds( )[-2:] ]) )
-        ###                        >>>--------tclean+deconvolve----------------------------------------------------------------------------------------^^^^^
-        self._status['stopcode'] = Div( text="<div>initial image</div>" )
-
         ###
         ### Initial Conditions
         ###
@@ -267,12 +301,16 @@ class InteractiveClean:
                      ### --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- ---
                      'initialize':      '''if ( ! convergence_src._initialized ) {
                                                convergence_src._initialized = true
+                                               convergence_src._window_closed = false
                                                window.addEventListener( 'beforeunload',
                                                                         function (e) {
-                                                                            ctrl_pipe.send( ids['stop'],
-                                                                                            { action: 'stop', value: { } },
-                                                                                              undefined ) }
-                                                                      )
+                                                                            // if the window is already closed this message is never
+                                                                            // delivered (unless interactive clean is called again then
+                                                                            // the event shows up in the newly created control pipe
+                                                                            if ( convergence_src._window_closed == false ) {
+                                                                                ctrl_pipe.send( ids['stop'],
+                                                                                                { action: 'stop', value: { } },
+                                                                                                  undefined ) } } )
                                            }''',
 
                      ### --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- ---
@@ -312,7 +350,8 @@ class InteractiveClean:
                                                        delete convergence_src._convergence_data
                                                        const pos = img_src.cur_chan
                                                        // fetch convergence information for the current channel (pos[1])
-                                                       conv_pipe.send( convergence_id, { action: 'update', value: pos[1] }, update_convergence )
+                                                       // ...convergence update expects [ stokes, chan ]
+                                                       conv_pipe.send( convergence_id, { action: 'update', value: pos }, update_convergence )
                                                    } else {
                                                        convergence_src._convergence_data = clean_msg.convergence
                                                        update_convergence( )
@@ -461,17 +500,6 @@ class InteractiveClean:
                                            }''',
                    }
 
-        self._cube = CubeMask( self._residual_path, mask=self._mask_path, abort=self._abort_handler )
-        ###
-        ### error or exception result
-        ###
-        self._error_result = None
-
-        ###
-        ### websocket servers
-        ###
-        self._control_server = None
-        self._converge_server = None
 
     def _init_pipes( self ):
         if not self.__pipes_initialized:
@@ -489,6 +517,10 @@ class InteractiveClean:
         ###
         self._init_pipes( )
 
+        self._status['log'] = Div( text='''<hr style="width:790px">%s''' % ''.join([ f'<p style="width:790px">{cmd}</p>' for cmd in self._clean.cmds( )[-2:] ]) )
+        ###                        >>>--------tclean+deconvolve----------------------------------------------------------------------------------------^^^^^
+        self._status['stopcode'] = Div( text="<div>initial image</div>" )
+
         ###
         ### Python-side handler for events from the interactive clean control buttons
         ###
@@ -695,7 +727,8 @@ class InteractiveClean:
                                                   //                { action: 'stop',
                                                   //                  value: { } },
                                                   //                update_gui )
-                                                  img_src.done( )
+                                                  convergence_src._window_closed = true
+                                                  img_src.done( )  /*** <<-------<<<< this will close the tab ***/
                                               } else if ( state.mode === 'continuous' &&
                                                           cb_obj.origin.name === 'stop' &&
                                                           ! state.awaiting_stop ) {
@@ -719,7 +752,7 @@ class InteractiveClean:
                                                              ),
                                                     code='''const pos = img_src.cur_chan;''' +             ### later we will receive the polarity from some widget mechanism...
                                                             self._js['update-converge'] + self._js['slider-update'] ) )
-        
+
         # Generates the HTML for the controls layout:
         # nmajor niter cycleniter cycle_factor threshold  -----------
         #                  slider                         -  image  -    help
@@ -822,11 +855,11 @@ class InteractiveClean:
 
         self._control_server = websockets.serve( self._pipe['control'].process_messages, self._pipe['control'].address[0], self._pipe['control'].address[1] )
         self._converge_server = websockets.serve( self._pipe['converge'].process_messages, self._pipe['converge'].address[0], self._pipe['converge'].address[1] )
-        resource_manager.reg_webserver(self._control_server.ws_server)
-        resource_manager.reg_webserver(self._converge_server.ws_server)
+        resource_manager( ).reg_webserver(self._control_server.ws_server)
+        resource_manager( ).reg_webserver(self._converge_server.ws_server)
         return async_loop( self._control_server, self._converge_server, self._cube.loop( ) )
 
-    def __call__( self, loop=asyncio.get_event_loop( ) ):
+    def __call__( self, loop=asyncio.new_event_loop( ) ):
         '''Display GUI using the event loop specified by ``loop``.
 
         Parameters
@@ -842,6 +875,14 @@ class InteractiveClean:
                                cell='12.0arcsec', specmode='cube',
                                interpolation='nearest', ... )( ) )
         '''
+        ###
+        ### new event loops must be set in asyncio to become active
+        ###
+        if loop != asyncio.get_event_loop( ):
+            asyncio.set_event_loop(loop)
+
+        self.__reset( )
+
         try:
             loop.run_until_complete(self.show( ))
             loop.run_forever( )
